import 'dart:ffi';

import 'package:flutter/material.dart';
import 'dart:async';

import 'package:flutter/services.dart';
import 'package:wl_base_help/wl_base_help.dart';
import 'package:wl_base_help/wl_clash_gen.dart';

void main() {
  runApp(const MyApp());
}

class MyApp extends StatefulWidget {
  const MyApp({super.key});

  @override
  State<MyApp> createState() => _MyAppState();
}

class _MyAppState extends State<MyApp> {
  String _platformVersion = 'Unknown';
  String _isRunAdminEerr = 'Unknown';
  String _isAdmin = 'no';
  String _isRunPassName = 'no';
  String isPPoe = 'no';
  final _wlBaseHelpPlugin = WlBaseHelp();

  @override
  void initState() {
    super.initState();
    initPlatformState();
  }

  Future<void> isDialUpEnabled() async {
    if(await _wlBaseHelpPlugin.isDialUpEnabled()){

      setState(() {
        isPPoe = "pppoe_yes";
      });
    } else {
      setState(() {
        isPPoe = "pppoe_no";
      });

    }
  }

  Future<void> initBuild() async {
    var baseConfig = BasicConfig(
        mixedPort: 9788,
        allowLan: false,
        bindAddress: '*',
        mode: ModeType.rule.name,
        logLevel: LogLevel.info.name,
        externalController: '127.0.0.1:9799',
        unifiedDelay: false,
        geodataMode: true,
        tcpConcurrent: true,
        findProcessMode: 'strict',
        globalClientFingerprint: 'chrome');

    var tunConfig = TunConfig(
        enable: false,
        stack: 'system',
        dnsHijack: ['198.18.0.2:53'],
        autoRoute: true,
        autoDetectInterface: true);
    var realityConfig = Reality(
        publicKey: 'dakfjlkdsjflsajlsdfa', shortID: 'sajdsaljlaslallasla');

    var wsOpts = WsOpts(
        path: '/path',
        headers: {'Host': 'v2ray.com'}
    );

    List<ProxyConfig> proxyConfg = [
      ProxyConfig(
          name: "测试",
          type: "ss",
          server: "1.1.1.1",
          port: 20202,
          password: 'fasla',
          udp: true,
          cipher: "aes-256-gmc"),
      ProxyConfig(
          name: "测试trojan",
          type: "trojan",
          server: "1.1.1.1",
          port: 20202,
          password: 'fadsdaa',
          udp: true,
          tls: true),
      ProxyConfig(
          name: "测试vless",
          type: "vless",
          server: "1.1.1.1",
          port: 20202,
          udp: true,
          tls: true,
          flow: 'xtls-rprx-vision',
          servername: 'www.yahoo.com',
          realityOpts: realityConfig,
          uuid: '001001001-0101010101-191919191919',
          clientFingerprint: 'chrome'),
      ProxyConfig(
          name: "测试vmess",
          type: "vmess",
          server: "1.1.1.1",
          port: 20202,
          udp: true,
          tls: true,
          uuid: '001001001-0101010101-191919191919',
          cipher: 'auto',
          alterId: 0,
          wsOpts: wsOpts,
      ),
    ];
    List<ProxyGroup> proxyGroups = [
      ProxyGroup(
          name: "proxy",
          type: "select",
          proxies: proxyConfg.map((e) => e.name).toList())
    ];
    var acitonProxy = proxyGroups.map((e) => e.name).first;

     List<Rule> rulesConfig = [
      Rule(type: 'GEOSITE', condition: 'geolocation-!cn', action: acitonProxy),
      Rule(type: 'GEOSITE', condition: 'cn', action: 'DIRECT'),
      Rule(type: 'MATCH', condition: null, action: acitonProxy),
     ];

    var defaultDns = ['119.29.29.29','223.5.5.5', '114.114.114.114'];
    var encDns =  ['tls://119.29.29.29', 'tls://223.5.5.5'];
    var proxyDns = ['8.8.8.8','tls://8.8.4.4#$acitonProxy', 'https://1.0.0.1/dns-query#$acitonProxy'];
    var dnsConfig = DNSConfig(
        enable: true,
        ipv6: false,
        enhancedMode: EnhancedMode.redirHost.name,
        defaultNameserver: defaultDns,
        nameserver: proxyDns,
        proxyServerNameserver: defaultDns,
        nameserverPolicy: {
          'www.baidu.com': defaultDns.first,
          'geosite:cn': defaultDns
        },
        listen: ':1035');
    var appConfig = AppConfigBuilder()
        .setBasicConfig(baseConfig)
        .setDNSConfig(dnsConfig)
        .setTunConfig(tunConfig)
        .setProxies(proxyConfg)
        .setProxyGroups(proxyGroups)
        .setRules(rulesConfig)
        .build();

    print("\n${appConfig.toYaml()}");
    print("\n${appConfig.toJson()}");
  }

  // Platform messages are asynchronous, so we initialize in an async method.
  Future<void> initPlatformState() async {
    String platformVersion;
    // Platform messages may fail, so we use a try/catch PlatformException.
    // We also handle the message potentially returning null.
    try {
      platformVersion =
          await _wlBaseHelpPlugin.getPlatformVersion() ?? 'Unknown platform version';
    } on PlatformException {
      platformVersion = 'Failed to get platform version.';
    }

    // If the widget was removed from the tree while the asynchronous platform
    // message was in flight, we want to discard the reply rather than calling
    // setState to update our non-existent appearance.
    if (!mounted) return;

    setState(() {
      _platformVersion = platformVersion;
    });
  }

  Future<void> onRunAdmin() async {
    String isRunadminErore;
    try {
      isRunadminErore = await _wlBaseHelpPlugin.runAsAdministrator() ?? 'error';
    } on PlatformException {
      isRunadminErore = 'error';
    }
    setState(() {
      _isRunAdminEerr = isRunadminErore;
    });
  }

  Future<void> onIsRunAdmin() async {
    bool isAdmin;
    try {
      isAdmin = await _wlBaseHelpPlugin.isRunningAsAdmin() ?? false;
    } on PlatformException {
      isAdmin = false;
      _isAdmin = 'error';
      setState(() {} );
    }
    setState(() {
      _isAdmin = isAdmin ? 'yes' : 'no';
    });
  }

  Future<void> onisProcessRunning(String passName) async {
    bool isRun;
    try {
      isRun = await _wlBaseHelpPlugin.isProcessRunning(passName) ?? false;
    } on PlatformException {
      isRun = false;
      _isRunPassName = 'error';
      setState(() {} );
    }
    setState(() {
      _isRunPassName = isRun ? 'yes' : 'no';
    });
  }

  Future<void> onkillProcess(String passName) async {

    try {
      await _wlBaseHelpPlugin.killProcess(passName);
    } on PlatformException {
      setState(() {} );
    }

  }
  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      home: Scaffold(
        appBar: AppBar(
          title: const Text('Plugin example app'),
        ),
        body: Center(
          child: Column(
            children: [
              Text('Running on: $_platformVersion\n'),
              const SizedBox(
                height: 20,
              ),
              // Text('Running on: $_isRunAdminEerr\n'),
              ElevatedButton(
                  onPressed: onRunAdmin, child: const Text("管理员重启")),
              const SizedBox(
                height: 20,
              ),
              ElevatedButton(
                  onPressed: onIsRunAdmin, child: Text("当前账号是否管理员:$_isAdmin")),
              const SizedBox(
                height: 20,
              ),
              ElevatedButton(
                  onPressed: () async{
                    await _wlBaseHelpPlugin.showConsole();
                  }, child: Text('显示控制台')),
              const SizedBox(
                height: 20,
              ),
              ElevatedButton(
                  onPressed: () async{
                    await _wlBaseHelpPlugin.hideConsole();
                  }, child: Text('隐藏控制台')),
              const SizedBox(
                height: 20,
              ),
              ElevatedButton(
                  onPressed: () async{
                    await onisProcessRunning('dart.exe');
                  }, child: Text(_isRunPassName)),
              const SizedBox(
                height: 20,
              ),
              ElevatedButton(
                  onPressed: () async{
                    await onkillProcess('IDMan.exe');
                  }, child: Text('隐藏控制台')),

              const SizedBox(
                height: 20,
              ),
              ElevatedButton(
                  onPressed: () async{
                    await initBuild();
                  }, child: Text('构建clash config')),
              const SizedBox(
                height: 20,
              ),
              ElevatedButton(
                  onPressed: () async{
                    await isDialUpEnabled();
                  }, child: Text(isPPoe)),
            ],
          ),
        ),
      ),
    );
  }
}
